Module: Grape::DSL::Parameters
- Included in:
- Validations::ParamsScope
- Defined in:
- lib/grape/dsl/parameters.rb
Overview
Defines DSL methods, meant to be applied to a ParamsScope, which define and describe the parameters accepted by an endpoint, or all endpoints within a namespace.
Defined Under Namespace
Classes: EmptyOptionalValue
Instance Method Summary collapse
-
#build_with(build_with) ⇒ Object
Set the module used to build the request.params.
-
#declared_param?(param) ⇒ Boolean
Test for whether a certain parameter has been defined in this params block yet.
-
#given(*attrs) { ... } ⇒ Object
Define a block of validations which should be applied if and only if the given parameter is present.
- #map_params(params, element, is_array = false) ⇒ Object
-
#optional(*attrs, **opts, &block) ⇒ Object
Allow, but don’t require, one or more parameters for the current endpoint.
-
#params(params) ⇒ Object
private
Hash of parameters relevant for the current scope.
-
#requires(*attrs, **opts, &block) ⇒ Object
(also: #group)
Require one or more parameters for the current endpoint.
-
#use(*names, **options) ⇒ Object
(also: #use_scope, #includes)
Include reusable params rules among current.
-
#with(*attrs) ⇒ Object
Define common settings for one or more parameters.
Instance Method Details
#build_with(build_with) ⇒ Object
Set the module used to build the request.params.
31 32 33 |
# File 'lib/grape/dsl/parameters.rb', line 31 def build_with(build_with) @api.inheritable_setting.namespace_inheritable[:build_params_with] = build_with end |
#declared_param?(param) ⇒ Boolean
Test for whether a certain parameter has been defined in this params block yet.
192 193 194 195 196 197 198 199 200 201 202 203 |
# File 'lib/grape/dsl/parameters.rb', line 192 def declared_param?(param) if lateral? # Elements of @declared_params of lateral scope are pushed in @parent. So check them in @parent. @parent.declared_param?(param) else # @declared_params also includes hashes of options and such, but those # won't be flattened out. @declared_params.flatten.any? do |declared_param_attr| first_hash_key_or_param(declared_param_attr.key) == param end end end |
#given(*attrs) { ... } ⇒ Object
Define a block of validations which should be applied if and only if the given parameter is present. The parameters are not nested.
181 182 183 184 185 186 187 |
# File 'lib/grape/dsl/parameters.rb', line 181 def given(*attrs, &) attrs.each do |attr| proxy_attr = first_hash_key_or_param(attr) raise Grape::Exceptions::UnknownParameter.new(proxy_attr) unless declared_param?(proxy_attr) end new_lateral_scope(dependent_on: attrs, &) end |
#map_params(params, element, is_array = false) ⇒ Object
209 210 211 212 213 214 215 216 217 218 219 220 221 |
# File 'lib/grape/dsl/parameters.rb', line 209 def map_params(params, element, is_array = false) if params.is_a?(Array) params.map do |el| map_params(el, element, true) end elsif params.is_a?(Hash) params[element] || (@optional && is_array ? EmptyOptionalValue : {}) elsif params == EmptyOptionalValue EmptyOptionalValue else {} end end |
#optional(*attrs, **opts, &block) ⇒ Object
Allow, but don’t require, one or more parameters for the current
endpoint.
141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 |
# File 'lib/grape/dsl/parameters.rb', line 141 def optional(*attrs, **opts, &block) type = opts[:type] opts = @group.deep_merge(opts) if instance_variable_defined?(:@group) && @group # check type for optional parameter group if attrs && block raise Grape::Exceptions::MissingGroupType if type.nil? raise Grape::Exceptions::UnsupportedGroupType unless Grape::Validations::Types.group?(type) end if opts[:using] require_optional_fields(attrs.first, opts) else validate_attributes(attrs, opts, &block) block ? new_scope(attrs, opts, true, &block) : push_declared_params(attrs, opts.slice(:as)) end end |
#params(params) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns hash of parameters relevant for the current scope.
226 227 228 229 230 |
# File 'lib/grape/dsl/parameters.rb', line 226 def params(params) params = @parent.params_meeting_dependency.presence || @parent.params(params) if instance_variable_defined?(:@parent) && @parent params = map_params(params, @element) if instance_variable_defined?(:@element) && @element params end |
#requires(*attrs, **opts, &block) ⇒ Object Also known as: group
Require one or more parameters for the current endpoint.
125 126 127 128 129 130 131 132 133 134 135 |
# File 'lib/grape/dsl/parameters.rb', line 125 def requires(*attrs, **opts, &block) opts[:presence] = { value: true, message: opts[:message] } opts = @group.deep_merge(opts) if instance_variable_defined?(:@group) && @group if opts[:using] require_required_and_optional_fields(attrs.first, opts) else validate_attributes(attrs, opts, &block) block ? new_scope(attrs, opts, &block) : push_declared_params(attrs, opts.slice(:as)) end end |
#use(*names, **options) ⇒ Object Also known as: use_scope, includes
Include reusable params rules among current. You can define reusable params with helpers method.
56 57 58 59 60 61 62 63 64 65 66 67 68 69 |
# File 'lib/grape/dsl/parameters.rb', line 56 def use(*names, **) named_params = @api.inheritable_setting.namespace_stackable_with_hash(:named_params) || {} names.each do |name| params_block = named_params.fetch(name) do raise "Params :#{name} not found!" end if .empty? instance_exec(, ¶ms_block) else instance_exec(**, ¶ms_block) end end end |
#with(*attrs) ⇒ Object
Define common settings for one or more parameters
163 164 165 166 |
# File 'lib/grape/dsl/parameters.rb', line 163 def with(*attrs, &) new_group_attrs = [@group, attrs.clone.first].compact.reduce(&:deep_merge) new_group_scope([new_group_attrs], &) end |