Class: Grape::Validations::ParamsScope

Inherits:
Object
  • Object
show all
Includes:
DSL::Parameters, ParamsDocumentation
Defined in:
lib/grape/validations/params_scope.rb

Defined Under Namespace

Classes: Attr

Constant Summary collapse

RESERVED_DOCUMENTATION_KEYWORDS =

There are a number of documentation options on entities that don’t have corresponding validators. Since there is nowhere that enumerates them all, we maintain a list of them here and skip looking up validators for them.

%i[as required param_type is_array format example].freeze
SPECIAL_JSON =
[JSON, Array[JSON]].freeze

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from ParamsDocumentation

#document_params

Methods included from DSL::Parameters

#build_with, #declared_param?, #given, #map_params, #optional, #params, #requires, #use, #with

Constructor Details

#initialize(opts) { ... } ⇒ ParamsScope

Open up a new ParamsScope, allowing parameter definitions per

Grape::DSL::Params.

Parameters:

  • opts (Hash)

    options for this scope

Options Hash (opts):

  • :element (Symbol)

    the element that contains this scope; for this to be relevant, @parent must be set

  • :element_renamed (Symbol, nil)

    whenever this scope should be renamed and to what, given nil no renaming is done

  • :parent (ParamsScope)

    the scope containing this scope

  • :api (API)

    the API endpoint to modify

  • :optional (Boolean)

    whether or not this scope needs to have any parameters set or not

  • :type (Class)

    a type meant to govern this scope (deprecated)

  • :type (Hash)

    group options for this scope

  • :dependent_on (Symbol)

    if present, this scope should only validate if this param is present in the parent scope

Yields:

  • the instance context, open for parameter definitions



64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/grape/validations/params_scope.rb', line 64

def initialize(opts, &block)
  @element          = opts[:element]
  @element_renamed  = opts[:element_renamed]
  @parent           = opts[:parent]
  @api              = opts[:api]
  @optional         = opts[:optional] || false
  @type             = opts[:type]
  @group            = opts[:group]
  @dependent_on     = opts[:dependent_on]
  @params_meeting_dependency = []
  @declared_params = []
  @index = nil

  instance_eval(&block) if block

  configure_declared_params
end

Instance Attribute Details

#elementObject

Returns the value of attribute element.



6
7
8
# File 'lib/grape/validations/params_scope.rb', line 6

def element
  @element
end

#indexObject

Returns the value of attribute index.



6
7
8
# File 'lib/grape/validations/params_scope.rb', line 6

def index
  @index
end

#params_meeting_dependencyObject (readonly)

Returns the value of attribute params_meeting_dependency.



7
8
9
# File 'lib/grape/validations/params_scope.rb', line 7

def params_meeting_dependency
  @params_meeting_dependency
end

#parentObject

Returns the value of attribute parent.



6
7
8
# File 'lib/grape/validations/params_scope.rb', line 6

def parent
  @parent
end

#typeObject (readonly)

Returns the value of attribute type.



7
8
9
# File 'lib/grape/validations/params_scope.rb', line 7

def type
  @type
end

Instance Method Details

#attr_meets_dependency?(params) ⇒ Boolean

Returns:

  • (Boolean)


110
111
112
113
114
115
# File 'lib/grape/validations/params_scope.rb', line 110

def attr_meets_dependency?(params)
  return true unless @dependent_on
  return false if @parent.present? && !@parent.attr_meets_dependency?(params)

  meets_hash_dependency?(params)
end

#brackets(val) ⇒ Object



149
150
151
# File 'lib/grape/validations/params_scope.rb', line 149

def brackets(val)
  "[#{val}]" if val
end

#configurationObject



82
83
84
# File 'lib/grape/validations/params_scope.rb', line 82

def configuration
  (@api.configuration.respond_to?(:evaluate) && @api.configuration.evaluate) || @api.configuration
end

#full_name(name, index: nil) ⇒ String

Returns the proper attribute name, with nesting considered.

Returns:

  • (String)

    the proper attribute name, with nesting considered.



135
136
137
138
139
140
141
142
143
144
145
146
147
# File 'lib/grape/validations/params_scope.rb', line 135

def full_name(name, index: nil)
  if nested?
    # Find our containing element's name, and append ours.
    "#{@parent.full_name(@element)}#{brackets(index || @index)}#{brackets(name)}"
  elsif lateral?
    # Find the name of the element as if it was at the same nesting level
    # as our parent. We need to forward our index upward to achieve this.
    @parent.full_name(name, index: @index)
  else
    # We must be the root scope, so no prefix needed.
    name.to_s
  end
end

#lateral?Boolean

A lateral scope is subordinate to its parent, but its keys are at the same level as its parent and thus is not contained within an element.

Returns:

  • (Boolean)

    whether or not this scope is lateral



167
168
169
# File 'lib/grape/validations/params_scope.rb', line 167

def lateral?
  @parent && !@element
end

#meets_dependency?(params, request_params) ⇒ Boolean

Returns:

  • (Boolean)


98
99
100
101
102
103
104
105
106
107
108
# File 'lib/grape/validations/params_scope.rb', line 98

def meets_dependency?(params, request_params)
  return true unless @dependent_on
  return false if @parent.present? && !@parent.meets_dependency?(@parent.params(request_params), request_params)

  if params.is_a?(Array)
    @params_meeting_dependency = params.flatten.filter { |param| meets_dependency?(param, request_params) }
    return @params_meeting_dependency.present?
  end

  meets_hash_dependency?(params)
end

#meets_hash_dependency?(params) ⇒ Boolean

Returns:

  • (Boolean)


117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
# File 'lib/grape/validations/params_scope.rb', line 117

def meets_hash_dependency?(params)
  # params might be anything what looks like a hash, so it must implement a `key?` method
  return false unless params.respond_to?(:key?)

  @dependent_on.each do |dependency|
    if dependency.is_a?(Hash)
      dependency_key = dependency.keys[0]
      proc = dependency.values[0]
      return false unless proc.call(params[dependency_key])
    elsif params[dependency].blank?
      return false
    end
  end

  true
end

#nested?Boolean

A nested scope is contained in one of its parent’s elements.

Returns:

  • (Boolean)

    whether or not this scope is nested



160
161
162
# File 'lib/grape/validations/params_scope.rb', line 160

def nested?
  @parent && @element
end

#required?Boolean

Returns whether or not this scope needs to be present, or can be blank.

Returns:

  • (Boolean)

    whether or not this scope needs to be present, or can be blank



173
174
175
# File 'lib/grape/validations/params_scope.rb', line 173

def required?
  !@optional
end

#reset_indexObject



177
178
179
# File 'lib/grape/validations/params_scope.rb', line 177

def reset_index
  @index = nil
end

#root?Boolean

Returns whether or not this scope is the root-level scope.

Returns:

  • (Boolean)

    whether or not this scope is the root-level scope



154
155
156
# File 'lib/grape/validations/params_scope.rb', line 154

def root?
  !@parent
end

#should_validate?(parameters) ⇒ Boolean

Returns whether or not this entire scope needs to be validated.

Returns:

  • (Boolean)

    whether or not this entire scope needs to be validated



88
89
90
91
92
93
94
95
96
# File 'lib/grape/validations/params_scope.rb', line 88

def should_validate?(parameters)
  scoped_params = params(parameters)

  return false if @optional && (scoped_params.blank? || all_element_blank?(scoped_params))
  return false unless meets_dependency?(scoped_params, parameters)
  return true if parent.nil?

  parent.should_validate?(parameters)
end