Module: Grape::DSL::Routing

Included in:
API::Instance
Defined in:
lib/grape/dsl/routing.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#endpointsObject (readonly)

Returns the value of attribute endpoints.



6
7
8
# File 'lib/grape/dsl/routing.rb', line 6

def endpoints
  @endpoints
end

Instance Method Details

#build_with(build_with) ⇒ Object



104
105
106
# File 'lib/grape/dsl/routing.rb', line 104

def build_with(build_with)
  inheritable_setting.namespace_inheritable[:build_params_with] = build_with
end

#cascade(value = nil) ⇒ Object



18
19
20
21
22
# File 'lib/grape/dsl/routing.rb', line 18

def cascade(value = nil)
  return inheritable_setting.namespace_inheritable.key?(:cascade) ? !inheritable_setting.namespace_inheritable[:cascade].nil? : true if value.nil?

  inheritable_setting.namespace_inheritable[:cascade] = value
end

#do_not_document!Object



122
123
124
# File 'lib/grape/dsl/routing.rb', line 122

def do_not_document!
  inheritable_setting.namespace_inheritable[:do_not_document] = true
end

#do_not_route_head!Object

Do not route HEAD requests to GET requests automatically.



109
110
111
# File 'lib/grape/dsl/routing.rb', line 109

def do_not_route_head!
  inheritable_setting.namespace_inheritable[:do_not_route_head] = true
end

#do_not_route_options!Object

Do not automatically route OPTIONS.



114
115
116
# File 'lib/grape/dsl/routing.rb', line 114

def do_not_route_options!
  inheritable_setting.namespace_inheritable[:do_not_route_options] = true
end

#given(conditional_option) ⇒ Object



8
9
10
11
12
# File 'lib/grape/dsl/routing.rb', line 8

def given(conditional_option, &)
  return unless conditional_option

  mounted(&)
end

#lint!Object



118
119
120
# File 'lib/grape/dsl/routing.rb', line 118

def lint!
  inheritable_setting.namespace_inheritable[:lint] = true
end

#mount(mounts, *opts) ⇒ Object



126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
# File 'lib/grape/dsl/routing.rb', line 126

def mount(mounts, *opts)
  mounts = { mounts => '/' } unless mounts.respond_to?(:each_pair)
  mounts.each_pair do |app, path|
    if app.respond_to?(:mount_instance)
      opts_with = opts.any? ? opts.first[:with] : {}
      mount({ app.mount_instance(configuration: opts_with) => path }, *opts)
      next
    end
    in_setting = inheritable_setting

    if app.respond_to?(:inheritable_setting, true)
      mount_path = Grape::Util::PathNormalizer.call(path)
      app.top_level_setting.namespace_stackable[:mount_path] = mount_path

      app.inherit_settings(inheritable_setting)

      in_setting = app.top_level_setting

      app.change!
      change!
    end

    # When trying to mount multiple times the same endpoint, remove the previous ones
    # from the list of endpoints if refresh_already_mounted parameter is true
    refresh_already_mounted = opts.any? ? opts.first[:refresh_already_mounted] : false
    if refresh_already_mounted && !endpoints.empty?
      endpoints.delete_if do |endpoint|
        endpoint.options[:app].to_s == app.to_s
      end
    end

    endpoints << Grape::Endpoint.new(
      in_setting,
      method: :any,
      path:,
      app:,
      route_options: { anchor: false },
      forward_match: !app.respond_to?(:inheritable_setting),
      for: self
    )
  end
end

#mounted(&block) ⇒ Object



14
15
16
# File 'lib/grape/dsl/routing.rb', line 14

def mounted(&block)
  evaluate_as_instance_with_configuration(block, lazy: true)
end

#namespace(space = nil, requirements: nil, **options, &block) ⇒ Object Also known as: group, resource, resources, segment

Declare a “namespace”, which prefixes all subordinate routes with its name. Any endpoints within a namespace, group, resource or segment, etc., will share their parent context as well as any configuration done in the namespace context.

Examples:


namespace :foo do
  get 'bar' do
    # defines the endpoint: GET /foo/bar
  end
end


221
222
223
224
225
226
227
228
229
# File 'lib/grape/dsl/routing.rb', line 221

def namespace(space = nil, requirements: nil, **options, &block)
  return Namespace.joined_space_path(inheritable_setting.namespace_stackable[:namespace]) unless space || block

  within_namespace do
    nest(block) do
      inheritable_setting.namespace_stackable[:namespace] = Grape::Namespace.new(space, requirements:, **options) if space
    end
  end
end

#prefix(prefix = nil) ⇒ Object

Define a root URL prefix for your entire API.



88
89
90
91
92
# File 'lib/grape/dsl/routing.rb', line 88

def prefix(prefix = nil)
  return inheritable_setting.namespace_inheritable[:root_prefix] if prefix.nil?

  inheritable_setting.namespace_inheritable[:root_prefix] = prefix.to_s
end

#route(methods, paths = ['/'], route_options = {}) ⇒ Object

Defines a route that will be recognized by the Grape API.

Examples:

Defining a basic route.

class MyAPI < Grape::API
  route(:any, '/hello') do
    {hello: 'world'}
  end
end

Parameters:

  • methods (HTTP Verb)

    One or more HTTP verbs that are accepted by this route. Set to ‘:any` if you want any verb to be accepted.

  • paths (String) (defaults to: ['/'])

    One or more strings representing the URL segment(s) for this route.



181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
# File 'lib/grape/dsl/routing.rb', line 181

def route(methods, paths = ['/'], route_options = {}, &)
  method = methods == :any ? '*' : methods
  endpoint_params = inheritable_setting.namespace_stackable_with_hash(:params) || {}
  endpoint_description = inheritable_setting.route[:description]
  all_route_options = { params: endpoint_params }
  all_route_options.deep_merge!(endpoint_description) if endpoint_description
  all_route_options.deep_merge!(route_options) if route_options.present?

  new_endpoint = Grape::Endpoint.new(
    inheritable_setting,
    method:,
    path: paths,
    for: self,
    route_options: all_route_options,
    &
  )
  endpoints << new_endpoint unless endpoints.include?(new_endpoint)

  inheritable_setting.route_end
  reset_validations!
end

#route_param(param, requirements: nil, type: nil) ⇒ Object

This method allows you to quickly define a parameter route segment in your API.

Parameters:

  • param (Symbol)

    The name of the parameter you wish to declare.

  • options (Hash)

    a customizable set of options



246
247
248
249
250
251
252
253
254
# File 'lib/grape/dsl/routing.rb', line 246

def route_param(param, requirements: nil, type: nil, **, &)
  requirements = { param.to_sym => requirements } if requirements.is_a?(Regexp)

  Grape::Validations::ParamsScope.new(api: self) do
    requires param, type: type
  end if type

  namespace(":#{param}", requirements:, **, &)
end

#routesObject

An array of API routes.



237
238
239
# File 'lib/grape/dsl/routing.rb', line 237

def routes
  @routes ||= endpoints.map(&:routes).flatten
end

#scope(_name = nil, &block) ⇒ Object

Create a scope without affecting the URL.

make the code more readable.

Parameters:

  • _name (Symbol) (defaults to: nil)

    Purely placebo, just allows to name the scope to



98
99
100
101
102
# File 'lib/grape/dsl/routing.rb', line 98

def scope(_name = nil, &block)
  within_namespace do
    nest(block)
  end
end

#version(*args, using: :path, cascade: true, parameter: 'apiver', strict: false, vendor: nil) { ... } ⇒ String?

Specify an API version.

Called without arguments, returns the most recently declared version (or nil). Called with one or more version strings, registers them and stores a VersionOptions value object on the inheritable settings; when given a block, the registration applies within a nested namespace.

Examples:

API with legacy support.

class MyAPI < Grape::API
  version 'v2'

  get '/main' do
    {some: 'data'}
  end

  version 'v1' do
    get '/main' do
      {legacy: 'data'}
    end
  end
end

Parameters:

  • args (Array<String, Symbol>)

    one or more version identifiers.

  • using (Symbol) (defaults to: :path)

    versioning strategy — one of :path (default), :header, :param, or :accept_version_header.

  • cascade (Boolean) (defaults to: true)

    forward to subsequent routes via the X-Cascade header on version mismatch. Defaults to true.

  • parameter (String) (defaults to: 'apiver')

    name of the query/body parameter that carries the version when using: :param. Defaults to ‘apiver’.

  • strict (Boolean) (defaults to: false)

    reject requests that don’t supply a usable version (header strategies). Defaults to false.

  • vendor (String, nil) (defaults to: nil)

    vendor segment for the :header strategy (+application/vnd.<vendor>-<version>+); required when using: :header.

Yields:

  • optional block to scope routes under this version.

Returns:

  • (String, nil)

    the most recently declared version.

Raises:



63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
# File 'lib/grape/dsl/routing.rb', line 63

def version(*args, using: :path, cascade: true, parameter: 'apiver', strict: false, vendor: nil, &block)
  return @versions&.last if args.empty?

  raise Grape::Exceptions::MissingVendorOption.new if using == :header && vendor.nil?

  requested_versions = args.flatten.map(&:to_s)
  options = VersionOptions.new(using:, cascade:, parameter:, strict:, vendor:)

  @versions = versions | requested_versions

  if block
    within_namespace do
      inheritable_setting.namespace_inheritable[:version] = requested_versions
      inheritable_setting.namespace_inheritable[:version_options] = options
      instance_eval(&block)
    end
  else
    inheritable_setting.namespace_inheritable[:version] = requested_versions
    inheritable_setting.namespace_inheritable[:version_options] = options
  end

  @versions&.last
end

#versionsObject

Returns array of defined versions.

Returns:

  • array of defined versions



257
258
259
# File 'lib/grape/dsl/routing.rb', line 257

def versions
  @versions ||= []
end