Exception: Grape::Exceptions::Validation
- Defined in:
- lib/grape/exceptions/validation.rb
Constant Summary
Constants inherited from Base
Instance Attribute Summary collapse
-
#message_key ⇒ Object
readonly
Returns the value of attribute message_key.
-
#params ⇒ Object
readonly
Returns the value of attribute params.
Attributes inherited from Base
Instance Method Summary collapse
-
#as_json(*_args) ⇒ Object
Remove all the unnecessary stuff from Grape::Exceptions::Base like status and headers when converting a validation error to json or string.
-
#initialize(params:, message: nil, status: nil, headers: nil) ⇒ Validation
constructor
A new instance of Validation.
Methods inherited from Base
Constructor Details
#initialize(params:, message: nil, status: nil, headers: nil) ⇒ Validation
Returns a new instance of Validation.
8 9 10 11 12 13 14 15 16 17 18 19 |
# File 'lib/grape/exceptions/validation.rb', line 8 def initialize(params:, message: nil, status: nil, headers: nil) @params = Array(params) if @message_key = case when Symbol then when Hash then [:key] end = () end super(status:, message:, headers:) end |
Instance Attribute Details
#message_key ⇒ Object (readonly)
Returns the value of attribute message_key.
6 7 8 |
# File 'lib/grape/exceptions/validation.rb', line 6 def @message_key end |
#params ⇒ Object (readonly)
Returns the value of attribute params.
6 7 8 |
# File 'lib/grape/exceptions/validation.rb', line 6 def params @params end |
Instance Method Details
#as_json(*_args) ⇒ Object
Remove all the unnecessary stuff from Grape::Exceptions::Base like status and headers when converting a validation error to json or string
23 24 25 |
# File 'lib/grape/exceptions/validation.rb', line 23 def as_json(*_args) to_s end |